@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
-__version__ = '1.7'
+__version__ = '1.8'
 __author__  = 'Olivier DUFAILLY'
 __license__ = 'BSD'
 
+
 import os
 import os.path
 import gzip
@@ -47,7 +48,6 @@ class switch(object):
         else:
             return False
 
-
 #
 # Binary Search in an ordered table (ascending order)
 #
@@ -65,6 +65,8 @@ def RemoveExt(filename):
   return filename[0:posext] if posext>0 else filename
 
 
+
+
 def cmp_to_key(mycmp):
     'Convert a cmp= function into a key= function'
     class K:
@@ -91,10 +93,17 @@ class UsageError(Exception):
   - User Input Errors
   - Missing actors, groups, ...
   '''
-  def __init__(self, msg, *args):
+  def __init__(self, msg, ret=C_OK, *args):
     super().__init__(msg)
+    self.ret = ret
     
-
+def RaiseCond(condition, msg, ret=C_OK, statusCell=None):
+  '''
+  Raise an UsageError Exception when 'condition' is True
+  Useful for non (really) structured coding.
+  '''
+  if condition:
+    raise UsageError(msg, statusCell, ret)
 
 class LogStatus:
   ''' Simple Logging class used to return statuses of complex algorithms '''
@@ -177,4 +186,4 @@ def version2float(v):
   ver += float(version[6]) / 1000000000000.
   return ver
 
-  
+
