@@ -9,9 +9,6 @@ from pypos3d.wftk.WFBasic import Point3d, Vector3d
 from pypos3d.pftk.PoserBasic import PoserToken, PoserConst, index, LowerLangName, LowerChanTypes
 from pypos3d.pftk.StructuredAttribut import ValueOpDelta
 from pypos3d.wftk.PoserFileParser import ParsingErrorException
-#           exc_type, exc_value, exc_tb = sys.exc_info()
-#           traceback.print_exception(exc_type, exc_value, exc_tb)
-
 
 
 def parseOrNull(s):
@@ -116,7 +113,9 @@ class FigurePart(LogStatus):
     vri = Vector3d()
     
     if self.rotation:
-      if self.rotation[0]=='(' and self.rotation[-1]==')': # Center is given by a Tuple of float
+      if isinstance(self.rotation, Vector3d):
+        vri = self.rotation  
+      elif self.rotation[0]=='(' and self.rotation[-1]==')': # Center is given by a Tuple of float
         try:
           vri = Vector3d(eval(self.rotation))
         except (SyntaxError, NameError, TypeError, ValueError) as e:
@@ -136,7 +135,9 @@ class FigurePart(LogStatus):
     bc = Point3d()
     
     if self.center:
-      if self.center[0]=='(' and self.center[-1]==')': # Center is given by a Tuple of float
+      if isinstance(self.center, Point3d):
+        bc = self.center
+      elif self.center[0]=='(' and self.center[-1]==')': # Center is given by a Tuple of float
         try:
           bc = Point3d(eval(self.center))
         except (SyntaxError, NameError, TypeError, ValueError, IndexError) as e:
@@ -167,6 +168,9 @@ class FigurePart(LogStatus):
     if not self.translation:
       return tr
     
+    if isinstance(self.translation, Vector3d):
+      return self.translation
+    
     # Cut the input string
     tabtrl = self.translation.split(':')
       
@@ -331,17 +335,19 @@ class ChannelDescriptor(LogStatus):
     self.lstOps = lstOps if lstOps else [ ]
     
 
-  def checkLink(self, fig, dirPathList=''):
+  def checkLink(self, fig=None, dirPathList=''):
     
-    p = self.actorName.find(':')
-    if p<0:
-      self.actorName = self.actorName + ':' + str(fig.getBodyIndex())
-
-    
-    self.act = fig.findActor(self.actorName)
-    if not self.act:
-      return self.status(PoserConst.C_ACTOR_NOTFOUND, "Unknown actor[{:s}] at column {:d}", self.actorName, self.no)
+    # When the Figure (fig) is None, the actor shall have been valued befaore the call
+    if fig:
+      p = self.actorName.find(':')
+      if p<0:
+        self.actorName = f'{self.actorName}:{fig.getBodyIndex()}'
       
+      self.act = fig.findActor(self.actorName)
+      if not self.act:
+        return self.status(PoserConst.C_ACTOR_NOTFOUND, "Unknown actor[{:s}] at column {:d}", self.actorName, self.no)
+    #else: # None Figure
+        
     # Check Channel Name (if the actor exists)
     self.gt = self.act.getChannel(self.chanName)
     if self.gt:
@@ -405,7 +411,7 @@ class ChannelDescriptor(LogStatus):
         try:
           MaterialName, NodeName, NodeInputName = self.groupName.split('.')
           
-          self.mat = fig.getMaterial(MaterialName)          
+          self.mat = fig.getMaterial(MaterialName) if fig else self.act.getMaterial(MaterialName)          
           if not self.mat:
             return self.status(PoserConst.C_MATERIAL_NOTFOUND, "Unknown material[{:s}] at column {:d}", MaterialName, self.no)
           
@@ -432,9 +438,11 @@ class ChannelDescriptor(LogStatus):
   
   
 
-  def analyseOps(self, fig):    
+  def analyseOps(self, fig=None):    
     ''' Check operations descriptions and replace string list by
     a list of ValueOp 
+    self.act shall be valued.
+    fig (Figure) can be None, in such case the operation will reference the C_NO_FIG parent
     '''
     ret = C_OK
     if not self.lstOps:
@@ -456,7 +464,7 @@ class ChannelDescriptor(LogStatus):
 
         ptind = qualifiedName.find('.')
         if ptind < 0:
-          figName = "Figure " + str(fig.getBodyIndex())
+          figName = f'Figure {fig.getBodyIndex()}' if fig else PoserConst.C_NO_FIG
           actName = self.act.getName()
           gtName = qualifiedName
         else:
@@ -464,9 +472,9 @@ class ChannelDescriptor(LogStatus):
 
           ptdp = qualifiedName.find(':')
           if (ptdp < 0):
-            actName = actName + ':' + str(fig.getBodyIndex())
+            actName = f'{actName}:{fig.getBodyIndex()}' if fig else f'{actName}'
 
-          figName = "Figure " + str(index(actName))
+          figName = f'Figure {index(actName)}' if fig else PoserConst.C_NO_FIG
           gtName = qualifiedName[ptind + 1:]
 
         #op = ValueOpDelta(PoserToken.E_valueOpKey, figName, actName, gtName, 0.0)
@@ -495,7 +503,8 @@ class ChannelImportAnalysis: # implements PoserConst
    
    When using alternateGeom on Victoria4, Victoria4.2 with Poser7 :
    - The alternateGeom channel starts at 1!!! (even if it should not)
-   - It's probably a Poser7 bug
+   - It's probably a Poser7 bug 
+     2022-09-08: Not a Bug, Just an option of genital parts to show or not
   '''
   # Operation keywords : Op, OpKey.Dep, valueKey.key, valueKey.val
   C_CMD_OP = "Op"
@@ -534,7 +543,7 @@ class ChannelImportAnalysis: # implements PoserConst
       if val:
         if (val.lower()==self.C_CMD_OP.lower()) and (val.lower()==self.C_CMD_OPKEY_DEP.lower()) \
             and (val.lower()==self.C_CMD_OPKEY_KEY.lower()) and (val.lower()==self.C_CMD_OPKEY_VAL.lower()):
-          logging.info("Unknown key word[" + self.tabChan[nolgn][0] + "] at line :" + nolgn)
+          logging.info(f"Unknown key word[{self.tabChan[nolgn][0]}] at line :{nolgn}")
           self.ts[nolgn][0] = C_FAIL
       else:
         logging.info("Missing Ligne Title[] at line :%d", nolgn)
