@@ -10,7 +10,8 @@ from langutil import C_OK, C_FAIL, C_ERROR, C_UNKNOWN, C_FILE_NOT_FOUND, UsageEr
 
 from pypos3d.wftk.PoserFileParser import PoserFileParser
 
-from pypos3d.pftk.PoserBasic import cleanName, getRealPath, index, PoserConst, PoserToken, Lang, create, WBoolLine
+from pypos3d.pftk.PoserBasic import cleanName, getRealPath, index, PoserConst, PoserToken, Lang, create, WBoolLine,\
+  RemoveQuotes
 from pypos3d.pftk.SimpleAttribut import LinkParmsSA, SimpleAttribut
 from pypos3d.pftk.StructuredAttribut import GenericTransform, ValueOpDelta, ChannelMorphStatus, ChannelMorphStatusList, PoserMaterial, Keys, calcMapping_KDTree, CustomData
 from pypos3d.pftk.GeomCustom import GeomCustom, FigureResFile
@@ -202,7 +203,7 @@ class Figure():
     return self._lstActor
 
   def getProps(self):
-    ''' Return a filter list of actors on 'Prop' descendant '''
+    ''' Return a filtered list of actors on 'Prop' descendant '''
     return [ p for p in self.getActors() if isinstance(p, PoserProp) ]
 
   def getRoot(self): return self._root
@@ -230,7 +231,8 @@ class Figure():
     if withIndex:
       return next( (a for a in self.getActors() if actorName == a.getName()), None)
     else:
-      return next( (a for a in self.getActors() if actorName == a.getName()[:-2]), None)
+      backoff = 1+len(str(self._bodyIdx))
+      return next( (a for a in self.getActors() if actorName == a.getName()[:-backoff]), None)
 
 
   def findProp(self, propName):
@@ -1179,7 +1181,13 @@ class Figure():
   #
   def importChannels(self, lstChanDest:list, poserRootDir, hideComputed=False):
     '''Import channel according to specifications contained by a list of ChannelDescriptor(s).
+    
+    ChannelDescriptors must be likned to target actors.
 
+    Old method entry kept for backward compatilbility.
+    
+    The real code of the method has been moved to PoserFile (to fix a small design issue)
+ 
     Parameters
     ----------
     lstChanDest : list
@@ -1191,60 +1199,10 @@ class Figure():
     hideComputed: boolean, optional, default False
       Hide computed parameters
     '''
-    ret = C_OK
-    for ccd in lstChanDest:
-      if ccd.ret == C_OK:
-        logging.info("Upgrading : " + ccd.act.getName())
-        gt = ccd.gt
-        if gt:
-          logging.info("Overriding Channel[" + gt.getName() + "]")
-        else:
-          gt = GenericTransform(ccd.chanType, ccd.chanName)
-          gt.setPrintName(ccd.printName)
-          ccd.act.addAttribut(gt)
-          logging.info("Creating Channel[" + gt.getName() + "]")
-
-        # Set standard values
-
-        # Do not override the initValue, if it has not been given
-        if ccd.initValue!=None:
-          gt.setInitValue(ccd.initValue)
-          gt.addKeyFrame(0, ccd.initValue)
-
-        gt.setMin(ccd.min)
-        gt.setTrackingScale(ccd.trackingScale)
-        gt.setForceLimits(1)
-        gt.setInterpStyleLocked(1)
-        gt.setHidden( (hideComputed and ccd.lstOps) or ccd.isHidden )
-
-        # Create the list of Alternate Geometries
-        if ccd.chanType==PoserToken.E_geomChan:
-          for fic in ccd.lstAltFiles:
-            ccd.act.addAltGeom(altGeomFile=fic, poserRootDir=poserRootDir)
-          ccd.max = len(ccd.act.getAltGeomList())
-
-        elif ccd.chanType==PoserToken.E_targetGeom: # Perform a Morph import
-          wg = readGeom(ccd.lstAltFiles[0].getCanonicalPath())
-          grp = wg.getGroup(ccd.groupName) if ccd.groupName else wg.getGroups()[0]
-          if not grp:
-            logging.warning("  Morph Group not found for Channel[%s] : %s %s", gt.getName(), ccd.lstAltFiles[0], ccd.groupName if ccd.groupName else 'default')
-            ret = min(ret, C_FAIL)
-            continue
-          ret = min(ret, ccd.act.createMorph(poserRootDir, grp, ccd.chanName, altGeomNo=ccd.altGeomNo)) # , minVectLen=minDist)
-
-        elif ccd.chanType==PoserToken.E_shaderNodeParm:
-          # Link the Material.Node.NodeInput.parmR to the new channel
-          ccd.ni.setParmR(ccd.chanName)
-
-        # The number of alternate geom, could have changed
-        gt.setMax(ccd.max)
-        for vod in ccd.lstOps:
-          logging.info("  Adding Op to Channel[" + gt.getName() + "] : " + vod.getPoserType().token + "(" + vod.l1 + "." + vod.l2 + ")")
-          gt.addVOD(vod)
-
-      else:
-        logging.info("Skipping channel:%s (%d)", ccd.chanName, ccd.no)
-    return ret
+    
+    return self._pf.importChannels(lstChanDest, poserRootDir, hideComputed)
+    
+    
 
   def getChannel(self, pa, expr):
     ''' Find the channel that matches the given expression.
@@ -1472,6 +1430,45 @@ class Figure():
   #   def adaptMagnet(self, poserRootDir, initialGeomFile):
   #     raise(Exception, "not Implemented yet")
 
+  def cleanShaderNode(self):
+    ''' Clean (delete) all shader nodes channels that are not referenced by any material input. 
+    
+    Manage also Props and their custom Materials
+    TODO: Create the same at PoserFile level and manage UNIVERSE attached props or Lights
+    
+    Returns
+    -------
+    int :
+      Ret Code is C_OK if nodes have been found and deleted, C_FAIL if none
+    '''
+    
+    body = self.getRootActor()
+    
+    # Retrieve all Figure ShaderNodes
+    shnNames = { shn.getName() for shn in [ p for p in body.getChannels() if p.getPoserType()==PoserToken.E_shaderNodeParm ] }
+
+    # Compute list of Referenced shadernodes in Figure's materials
+    usedShnNames = PoserMaterial.getShaderNodesFromList(self.getLstMaterial())
+
+    # Scan Props with custom Materials
+    for pp in [ p for p in self.getProps() if p.getCustomMaterial() ]:
+      usedShnNames |= PoserMaterial.getShaderNodesFromList( [ m for m in pp.getLstAttr() if (m.getPoserType()==PoserToken.E_material) ] )
+
+    logging.info(f"Figure [{self.getPrintName()}]: Found ShaderNodes {len(shnNames)}")
+
+    shnNames -= usedShnNames
+    
+    # Filter the set of shadernodes names before deletion
+    lstShn = [ shn for shn in [ p for p in body.getChannels() if p.getPoserType()==PoserToken.E_shaderNodeParm and p.getName() in shnNames ] ]
+
+    logging.info(f"Figure [{self.getPrintName()}]: Cleaning {len(lstShn)} ShaderNodes ")
+    
+    for gt in lstShn:
+      logging.info(f"Figure [{self.getPrintName()}] Removing Channel {gt.getName()} cleared in {body.getName()}")
+      body.deleteChannel(gt)
+      
+    return C_OK if lstShn else C_FAIL
+  
 
   def attachActor(self, parent:'new Parent Actor', srcAct:'Actor to attach from another figure', changeRefToBODY=True,\
                   withProp=True, withDeformer=False):
