@@ -293,6 +293,9 @@ class PoserConst(object):
   C_NO_NODE = "NO_NODE"
   C_NO_MAPG = '"NO_MAP"'
   C_NO_MAP = "NO_MAP"
+  
+  # No figure string (for channels operation)
+  C_NO_FIG = '_NO_FIG_'
 
   # Shader Node Code
   kNodeTypeCodeBLENDER  = 1000
