@@ -4,7 +4,7 @@ import os, gzip
 
 from langutil import C_OK, C_FAIL, C_ERROR, version2float
 from pypos3d.wftk.PoserFileParser import PoserFileParser, PZ3_FNAT
-
+from pypos3d.wftk.WaveGeom import readGeom
 from pypos3d.pftk.PoserBasic import getRealPath, index, PoserConst, PoserToken , isCompressed, Lang, getPoserFileType, create, PoserWriter
 from pypos3d.pftk.StructuredAttribut import GenericTransform, NodeInput, PoserMaterial
 from pypos3d.pftk.GeomCustom import GeomCustom
@@ -1242,6 +1242,76 @@ class PoserFile(object):
     logging.info("Figure Created: [%s]:%d", name if name else '', bodyIdx)
     return fig
 
+  def importChannels(self, lstChanDest:list, poserRootDir, hideComputed=False):
+    '''Import channel according to specifications contained by a list of ChannelDescriptor(s).
+    
+    ChannelDescriptors must be likned to target actors.
+
+    Parameters
+    ----------
+    lstChanDest : list
+      list of ChannelDescriptor
+
+    poserRootDir : str
+      Location of Poser files
+
+    hideComputed: boolean, optional, default False
+      Hide computed parameters
+    '''
+    ret = C_OK
+    for ccd in lstChanDest:
+      if ccd.ret == C_OK:
+        logging.info("Upgrading : " + ccd.act.getName())
+        gt = ccd.gt
+        if gt:
+          logging.info("Overriding Channel[" + gt.getName() + "]")
+        else:
+          gt = GenericTransform(ccd.chanType, ccd.chanName)
+          gt.setPrintName(ccd.printName)
+          ccd.act.addAttribut(gt)
+          logging.info("Creating Channel[" + gt.getName() + "]")
+
+        # Set standard values
+
+        # Do not override the initValue, if it has not been given
+        if ccd.initValue!=None:
+          gt.setInitValue(ccd.initValue)
+          gt.addKeyFrame(0, ccd.initValue)
+
+        gt.setMin(ccd.min)
+        gt.setTrackingScale(ccd.trackingScale)
+        gt.setForceLimits(1)
+        gt.setInterpStyleLocked(1)
+        gt.setHidden( (hideComputed and ccd.lstOps) or ccd.isHidden )
+
+        # Create the list of Alternate Geometries
+        if ccd.chanType==PoserToken.E_geomChan:
+          for fic in ccd.lstAltFiles:
+            ccd.act.addAltGeom(altGeomFile=fic, poserRootDir=poserRootDir)
+          ccd.max = len(ccd.act.getAltGeomList())
+
+        elif ccd.chanType==PoserToken.E_targetGeom: # Perform a Morph import
+          wg = readGeom(ccd.lstAltFiles[0].getCanonicalPath())
+          grp = wg.getGroup(ccd.groupName) if ccd.groupName else wg.getGroups()[0]
+          if not grp:
+            logging.warning("  Morph Group not found for Channel[%s] : %s %s", gt.getName(), ccd.lstAltFiles[0], ccd.groupName if ccd.groupName else 'default')
+            ret = min(ret, C_FAIL)
+            continue
+          ret = min(ret, ccd.act.createMorph(poserRootDir, grp, ccd.chanName, altGeomNo=ccd.altGeomNo)) # , minVectLen=minDist)
+
+        elif ccd.chanType==PoserToken.E_shaderNodeParm:
+          # Link the Material.Node.NodeInput.parmR to the new channel
+          ccd.ni.setParmR(ccd.chanName)
+
+        # The number of alternate geom, could have changed
+        gt.setMax(ccd.max)
+        for vod in ccd.lstOps:
+          logging.info("  Adding Op to Channel[" + gt.getName() + "] : " + vod.getPoserType().token + "(" + vod.l1 + "." + vod.l2 + ")")
+          gt.addVOD(vod)
+
+      else:
+        logging.info("Skipping channel:%s (%d)", ccd.chanName, ccd.no)
+    return ret
 
 
 
