@@ -237,6 +237,14 @@ class PoserMeshedObject(StructuredAttribut):
   def getChannel(self, channelName):
     return self._channels.get(channelName.lower())
 
+  def getChannelByPrintName(self, channelPrintName):
+    for gt in self._channels.values():
+      if gt.getPrintName()==channelPrintName:
+        return gt
+    
+    return None
+
+
   # Find the GenericTransform that represents a TargetGeom and that has the
   # given "channelName"
   # @param channelName
