@@ -196,7 +196,7 @@ class ValueOpDelta(SimpleAttribut):
           self.addValueKey(t[0], t[1])
 
 
-    elif pfigure and pactor and channelExpr:
+    elif pactor and channelExpr: # Try to support None Figure. elif pfigure and pactor and channelExpr:
       # String Cleaning
       expr = channelExpr.strip()
   
@@ -206,7 +206,7 @@ class ValueOpDelta(SimpleAttribut):
       if (expr[-1]=="'") or (expr[-1]=="\""):
         expr = expr[:len(expr) - 1]
   
-      if (len(expr) < 2):
+      if len(expr) < 2:
         logging.info("Too short string")
         return
   
@@ -239,17 +239,17 @@ class ValueOpDelta(SimpleAttribut):
   
       ptind = qualifiedName.find('.')
       if ptind < 0:
-        self.l0 = f"Figure {pfigure.getBodyIndex()}"
+        self.l0 = f'Figure {pfigure.getBodyIndex()}' if pfigure else PoserConst.C_NO_FIG
         self.l1 = pactor.getName()
         self.l2 = qualifiedName
       else:
         self.l1 = qualifiedName[0:ptind]
         
         ptdp = qualifiedName.find(':')
-        if (ptdp < 0):
-          self.l1 += f':{pfigure.getBodyIndex()}'
+        if ptdp < 0:
+          self.l1 += f':{pfigure.getBodyIndex()}' if pfigure else ''
   
-        self.l0 = "Figure " + str(index(self.l1))
+        self.l0 = f'Figure {index(self.l1)}' if pfigure else PoserConst.C_NO_FIG
         self.l2 = qualifiedName[ptind + 1:]
   
     elif src: # Copy Constructor
