@@ -337,9 +337,6 @@ class NodeInput(StructuredAttribut):
       fw.write(f"{nextPfx}exposedAs \"{self._exposedAs}\"\n")
     fw.write(f"{nextPfx}}}\n")
 
-
-
-
   def getFile(self): return self._file
 
   #
@@ -360,6 +357,21 @@ class NodeInput(StructuredAttribut):
   def setParmR(self, s):
     self._parmR = s
 
+  def setParmG(self, s):
+    self._parmG = s
+
+  def setParmB(self, s):
+    self._parmB = s
+
+  def getParmR(self):
+    return self._parmR
+
+  def getParmG(self):
+    return self._parmG
+  
+  def getParmB(self):
+    return self._parmB
+
   def getInNode(self, sht):
     if self._node and self._node!=PoserConst.C_NO_NODE:
       return sht.getNodeByInternalName(self._node)
@@ -534,17 +546,18 @@ class ShaderTree(StructuredAttribut):
   #					node NO_NODE
   #					file NO_MAP
   #					}
-  def CreateNode(self, poserkNodeTypeCode, pos=None, inputsCollapsed=False, showPreview=False): # kNodeTypeCodeBLENDER or kNodeTypeCodeIMAGEMAP
+  def CreateNode(self, poserkNodeTypeCode, inpBaseName=None, pos=None, inputsCollapsed=False, showPreview=False): # kNodeTypeCodeBLENDER or kNodeTypeCodeIMAGEMAP
     n = Node()
     # Generated a name according to the type
-    baseName = 'Blender_' if poserkNodeTypeCode==PoserConst.kNodeTypeCodeBLENDER else 'Image_Map_'
-
-    lstno = [ nodeNameNo(no._nodeName) for no in self.getLstAttr() \
-      if no.getNodeType()==poserkNodeTypeCode ]
-
-    no = max(lstno) + 1 if lstno else 0
-
-    n._name = baseName+str(no)
+    if inpBaseName:
+      n._name = inpBaseName
+    else:
+      lstno = [ nodeNameNo(no._nodeName) for no in self.getLstAttr() \
+        if no.getNodeType()==poserkNodeTypeCode ]
+  
+      no = max(lstno) + 1 if lstno else 0
+      n._name  = ('Blender_' if poserkNodeTypeCode==PoserConst.kNodeTypeCodeBLENDER else 'Image_Map_') + str(no)
+  
     n._nodeName = n._name
     n._typeName = 'blender' if poserkNodeTypeCode==PoserConst.kNodeTypeCodeBLENDER else 'image_map'
 
@@ -572,7 +585,41 @@ class ShaderTree(StructuredAttribut):
     ni = nodeDest.getInputByInternalName(inpName=inputName)
     ni._node = nodeRef.getName()
 
+  def RemoveNodeNonRec(self, n):
+    ''' Delete a node from the tree. Without linkage check nor reccursive delete. '''
+    r = self.deleteAttribut(n.getName())
+    if not r:
+      logging.warning("Node:%s not found in ShaderTree",n.getName())
+    
+  def __RecRemoveNode(self, n, shNodeSet):
+    ''' Internal Recursively Delete a node from the tree. With valueParm linkage '''
+    # For each child node
+    if not n: return 
+    
+    lsrc = n.getLstAttr()
+    if lsrc:
+      for ni in [ noi for noi in lsrc if isinstance(noi, NodeInput)]:
+        innode = ni.getInNode(self)
+        # Store potential Attached shaderNodes
+        for parmS in [ ni.getParmR(), ni.getParmG(), ni.getParmB() ]:
+          if parmS and isinstance(parmS, str):
+            shNodeSet.add(RemoveQuotes(parmS))
 
+        self.__RecRemoveNode(innode, shNodeSet)
+    
+    r = self.deleteAttribut(n.getName())
+    if not r:
+      logging.warning("Node:%s not found in ShaderTree",n.getName())
+    
+    
+  def RemoveNode(self, n):
+    ''' Recursively Delete a node from the tree. With valueParm linkage.
+    Return:
+      set of shaderNodes Names to delete
+    '''
+    resSet = set()
+    self.__RecRemoveNode(n, resSet)
+    return resSet
 
 #
 class Output(StructuredAttribut):
@@ -708,6 +755,40 @@ class PoserMaterial(StructuredAttribut):
     st = self._shaderTree
     return [ po for po in st.getLstAttr() if isinstance(po, Node) ] if st else [ ]
 
+  def getShaderNodes(self):
+    ''' Compute the list of ShaderNodes names of a PoserMaterials.
+    
+    Returns:
+    --------
+      a set of shadernodes names
+    '''
+    usedShnNames = set()
+    
+    for node in self.getLstNodes():
+      for ni in [ attr for attr in node.getLstAttr() if attr.getPoserType()==PoserToken.E_nodeInput ]:
+        usedShnNames |= { RemoveQuotes(ni.getParmR()), RemoveQuotes(ni.getParmG()), RemoveQuotes(ni.getParmB()) }  
+
+    return usedShnNames
+
+
+  @classmethod
+  def getShaderNodesFromList(cls, lstMat):
+    ''' Compute the list of ShaderNodes names from a list of PoserMaterials.
+    
+    Returns:
+    --------
+      a set of shadernodes names
+      
+    '''
+    usedShnNames = set()
+    
+    for mat in lstMat:
+      for node in mat.getLstNodes():
+        for ni in [ attr for attr in node.getLstAttr() if attr.getPoserType()==PoserToken.E_nodeInput ]:
+          usedShnNames |= { RemoveQuotes(ni.getParmR()), RemoveQuotes(ni.getParmG()), RemoveQuotes(ni.getParmB()) }  
+
+    return usedShnNames
+
   def setDirect(self, tokenID, val):
     if tokenID == PoserToken.E_KdColor:
       tv = val.split()
@@ -971,6 +1052,9 @@ class Keys(StructuredAttribut):
   #
   #  Return the list of declared "k" frames.
   #
+  def getKeyList(self):
+    return [ ks for ks in sorted(self._hshKeys.values(), key=lambda k:k._noFrame) ]
+  
   def getKey(self, keyno=0): return self._hshKeys[keyno]
 
   #
@@ -2035,8 +2119,11 @@ class CustomData(StructuredAttribut):
       code,cn,rw = st.getLine()
       if code== PoserFileParser.TT_WORD:
         if cn=="data":
-          tv = rw.split()
-          self.set(tv[0], RemoveQuotes(tv[3]), int(tv[1]), int(tv[2]))
+          # FIX: split generate errors when the string data are containing spaces
+          pg = rw.find('"')
+          sdata = rw[pg:]
+          tv = rw[:pg-1].split()
+          self.set(tv[0], RemoveQuotes(sdata), int(tv[1]), int(tv[2]))
         else: #  Mot illegal
           logging.info("Line[%s] - Not accepted word:%s",st.lineno(), cn)
       elif (code==PoserFileParser.TT_RIGHTBRACKET) or (code==PoserFileParser.TT_EOF):
@@ -2046,7 +2133,7 @@ class CustomData(StructuredAttribut):
     # End of read
 
   def write(self, fw, pfx):
-    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
+    fw.write(f"{pfx}customData\n")
     nextPfx = pfx + "\t"
     fw.write(f"{nextPfx}{{\n")
 
