@@ -858,7 +858,7 @@ class GeomGroup(object):
           if bigFaceText: # Since bigFaceText is a best case, we add all or none texture coordinates
             p0.texture = self.texList[edgeDesc[6]]
             p1.texture = self.texList[edgeDesc[7]]
-          bigFace.append( (p0, p1) )
+          bigFace.append( Edge(p0, p1) )
     
     self.addFaceByEdges(bigFace, bigFaceText, newMatName)
     # Remove old faces
