@@ -175,6 +175,23 @@ def SArcSin(a,b):
   else:
     return 1.0
 
+def readPoint(s):
+  ''' Convert a string into a Point3d instance.
+  The string shall contain three comma separated floats.
+  This function may raise exceptions
+  '''
+  s = s.strip()
+  return Point3d(eval(s)) if s else Point3d() 
+  
+def readVector(s):
+  ''' Convert a string into a Vector3d instance.
+  The string shall contain three comma separated floats.
+  This function may raise exceptions
+  '''
+  s = s.strip()
+  return Vector3d(eval(s)) if s else Vector3d()
+  
+
 
 # Used as base Object for Vector3d
 class Point3d(object):
@@ -943,15 +960,29 @@ def Regularity(p00, p01, p10, p11):
 # ==============================================================================
 # Edge Management
 # ==============================================================================
-class Edge(NamedTuple):
+class Edge: # (NamedTuple):
   ''' The class represents an edge, composed of 2 Point3d.
   The edge may also contains the indexes of the Point3d in a coordinate list.
   '''
-  p0: Point3d
-  p1: Point3d
-  idx0: int = -1
-  idx1: int = -1
-  hasTexture: bool = False
+  # p0: Point3d
+  # p1: Point3d
+  # idx0: int = -1
+  # idx1: int = -1
+  # hasTexture: bool = False
+ 
+  def __init__(self, p0, p1, idx0=-1, idx1=-1, hasTexture=False):
+    self.p0 = p0
+    self.p1 = p1
+    self.idx0 = idx0
+    self.idx1 = idx1
+    
+    try:
+      getattr(p0, 'texture')
+      getattr(p1, 'texture')
+      self.hasTexture = True
+      
+    except AttributeError:
+      self.hasTexture = hasTexture
  
   def hashCode(self):
     return (min(self.idx0, self.idx1) << 32) | max(self.idx0, self.idx1)
@@ -1441,7 +1472,7 @@ class CoordSyst:
     coordList = [ ]
   
     for edge in lstEdges:      
-      IndexAdd(coordList, edge[0])
+      IndexAdd(coordList, edge.p0)
 
     return self.RadialScalePoint(coordList, k)
 
@@ -2597,7 +2628,7 @@ def CreateLoop(lstEdges):
   BestFaceLst = []
   
   # EdgeStock is a list of Edge Descriptor [ idx0, idx1 ]
-  EdgeStock = [ [ IndexAdd(coordList, edge[0]), IndexAdd(coordList, edge[1]) ] for edge in lstEdges ]      
+  EdgeStock = [ [ IndexAdd(coordList, edge.p0), IndexAdd(coordList, edge.p1) ] for edge in lstEdges ]      
 
   # Maximum Result : 1 face with all edges
   MaxScore = len(EdgeStock)
