@@ -170,6 +170,7 @@ class WaveGeom(object):
 
 
   def _selfInit(self):
+    # Name of file
     self._name= ''
 
     #  The vertex list shared between groups
@@ -194,19 +195,23 @@ class WaveGeom(object):
 
     self._curWriteMatIdx = 0
 
-    # Name of file
+    # Name of optional Material file and associated dict of materials
     self._libMaterialName = ''
     self.libMat = {}
 
   def __init__(self, st=None, filename=None, usemtl=False, imgdirpath='.'):
     ''' Create a Wavefront geometry.
-    Initialize temporaly lists for reading operations.
+    Initialize temporary lists for reading operations.
+    
     Parameters
     ----------
     st : optional input file 
       if None, Create an empty WaveGeom with a first default internal group 
       named "default".
       
+    filename : optional, str, default is None
+      file to set in 'Name' property. Not used for loading
+      
     usemtl : optional, bool, default is False
       Indicate if material files (*.mtl) shall be read
       
@@ -245,8 +250,6 @@ class WaveGeom(object):
       # FIX: Also required when the material list is empty at the end of the readInternal() call
       self.addMaterial("default") 
       
-    #except Exception as e: # Should
-    #  logging.error("Error:%s", str(e))
 
   def setName(self, n):
     self._name = n
@@ -2096,7 +2099,7 @@ def PlaneCut(target, centerOrRepOrPlane, eu=None, ev=None, materialName='Section
     
       # Add the new Edges to the closing Face
       if lstNewEdges:
-        closingFaceVx += lstNewEdges
+        closingFaceVx += [ Edge(e[0], e[1]) for e in lstNewEdges ]
         closingFaceText &= hasTexture
 
   
@@ -2290,7 +2293,7 @@ def PlaneSplit(target, centerOrRepOrPlane, eu=None, ev=None, radialLimit=0.0, ma
     
     # Add the new Edges to the closing Face
     if lstNewEdges:
-      closingFaceVx += lstNewEdges
+      closingFaceVx += [ Edge(e[0], e[1]) for e in lstNewEdges ]
       closingFaceText &= hasTexture
 
   # Finish the group's creation : Add each face to the group
@@ -2408,10 +2411,6 @@ def PlaneSlice(target, centerOrRepOrPlane, eu=None, ev=None, cutFaceMatLst=None,
     fattr = fao.fAttr
     hasTexture = (fattr & C_FACE_TEXT)!=0
     
-    # if lastidx-startidx<3:
-    #   if WFBasic.PYPOS3D_TRACE: print(f'PlaneSlice[{target.getName()}] ignoring NON face: {faceno}')
-    #   continue
-    
     # Compute the list of enhanced Vertex, without aligned ones
     # We have at least 3 points
     prevPt = Point3d(coordList[target.vertIdx[startidx]])
@@ -2456,7 +2455,7 @@ def PlaneSlice(target, centerOrRepOrPlane, eu=None, ev=None, cutFaceMatLst=None,
  
     # Add the new Edges to the closing Face
     if lstNewEdges:
-      closingFaceVx += lstNewEdges
+      closingFaceVx += [ Edge(e[0], e[1]) for e in lstNewEdges ]
       if keepMaterial:
         cutFaceMatLst.append(fattr & C_FACE_MATMASK)
       
@@ -2481,7 +2480,7 @@ def PlaneSlice(target, centerOrRepOrPlane, eu=None, ev=None, cutFaceMatLst=None,
         pmid = Point3d(e.p0).add(e.p1).scale(0.5)
         loccl.append(pmid)
         
-        # Compute potential texute coordinate
+        # Compute potential texture coordinate
         if hasTexture:
           pmid.texture = TexCoord2f(TexCoord2f(e.p0.texture).add(e.p1.texture).scale(0.5))
         
